/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2004 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Id: isdsl.c,v 1.5 2004/04/26 22:48:33 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <regex.h>
#include "isdsl.h"

char dslregex[] = "^[^.0-9]*[0-9]{1,3}-[0-9]{1,3}-[0-9]{1,3}-[0-9]{1,3}[^0-9]*.";

/* int main()
{
    // char string[] = "200-204-48-99.dial-up.telesp.net.br";
    char string[] = "www.wizard.ca";
    printf("isDSL(%s): %d\n", string, isDSL(string));
}
*/

int isDSL(char *address) 
{
    regex_t preg;
    char regerrbuf[80];
    int ret;

    if (address == NULL) {
        return (0);
    }

    ret = regcomp(&preg, dslregex, REG_EXTENDED | REG_NOSUB);
     
    if (ret != 0) {
        /* problem with regular expression */
        regerror(ret, &preg, regerrbuf, sizeof(regerrbuf));
        syslog(LOG_INFO, "Warning: isDSL(): regcomp(): %s\n", regerrbuf);
    } else {
        /* successfully compiled */
        if (regexec(&preg, address, 0, NULL, 0) == 0) {
            /* match */
            regfree(&preg);

            return (1);
        } else {
            /* no match */
        }
    }
    regfree(&preg);

    return (0); /* does not match, or error */
}

